------------------------------------------------------------------------
--  MODULE: Music track replacement for DOOM
------------------------------------------------------------------------
--
--  Copyright (C) 2014 Andrew Apted
--  Copyright (C) 2020 Simon Volpert
--
--  This program is free software; you can redistribute it and/or
--  modify it under the terms of the GNU General Public License
--  as published by the Free Software Foundation; either version 2
--  of the License, or (at your option) any later version.
--
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--  GNU General Public License for more details.
--
------------------------------------------------------------------------

JUKEBOX = {}

JUKEBOX.doom1_music =
{
  "E1M1", "E1M2", "E1M3",
  "E1M4", "E1M5", "E1M6",
  "E1M7", "E1M8", "E1M9",
  "E2M1", "E2M2", "E2M3",
  "E2M4", "E2M5", "E2M6",
  "E2M7", "E2M8", "E2M9",
  "E3M1", "E3M2", "E3M3",
  "E3M4", "E3M5", "E3M6",
  "E3M7", "E3M8", "E3M9",
  "E4M1", "E4M2", "E4M3",
  "E4M4", "E4M5", "E4M6",
  "E4M7", "E4M8", "E4M9"
}


JUKEBOX.doom2_music =
{
  "RUNNIN", "STALKS", "COUNTD", "BETWEE", "DOOM",
  "THE_DA", "SHAWN",  "DDTBLU", "IN_CIT", "DEAD",
  "STLKS2", "THEDA2", "DOOM2",  "DDTBL2", "RUNNI2",
  "DEAD2",  "STLKS3", "ROMERO", "SHAWN2", "MESSAG",
  "COUNT2", "DDTBL3", "AMPIE",  "THEDA3", "ADRIAN",
  "MESSG2", "ROMER2", "TENSE",  "SHAWN3", "OPENIN",
  "EVIL", "ULTIMA"
}

JUKEBOX.heretic_music =
{
  "E1M1", "E1M2", "E1M3",
  "E1M4", "E1M5", "E1M6",
  "E1M7", "E1M8", "E1M9",
  "E2M1", "E2M2", "E2M3",
  "E2M4", "E1M4", "E2M6",
  "E2M7", "E2M8", "E2M9",
  "E1M1", "E3M2", "E3M3",
  "E1M6", "E1M3", "E1M2",
  "E1M5", "E1M9", "E2M6",
  "E1M6", "E1M2", "E1M3",
  "E1M4", "E1M5", "E1M1",
  "E1M7", "E1M8", "E1M9",
  "E2M1", "E2M2", "E2M3",
  "E2M4", "E1M4", "E2M6",
  "E2M7", "E2M8", "E2M9"
}
JUKEBOX.heretic_levels =
{
  "E1M1", "E1M2", "E1M3",
  "E1M4", "E1M5", "E1M6",
  "E1M7", "E1M8", "E1M9",
  "E2M1", "E2M2", "E2M3",
  "E2M4", "E2M5", "E2M6",
  "E2M7", "E2M8", "E2M9",
  "E3M1", "E3M2", "E3M3",
  "E3M4", "E3M5", "E3M6",
  "E3M7", "E3M8", "E3M9",
  "E4M1", "E4M2", "E4M3",
  "E4M4", "E4M5", "E4M6",
  "E4M7", "E4M8", "E4M9",
  "E5M1", "E5M2", "E5M3",
  "E5M4", "E5M5", "E5M6",
  "E5M7", "E5M8", "E5M9"
}

JUKEBOX.search_dir = "data/midi"
JUKEBOX.script_dir = "data/acs"

function JUKEBOX.insert_music(self)
  local music_types = {"mid", "it"}
  local music_list
  local music_prefix = "D_"
  if (OB_CONFIG.game == "doom1") or (OB_CONFIG.game == "ultdoom") then
    music_list = JUKEBOX.doom1_music
  elseif (OB_CONFIG.game == "heretic") then
    music_list = JUKEBOX.heretic_music
    music_prefix = "MUS_"
  else
    music_list = JUKEBOX.doom2_music
  end
  local music_dir = {}
  for _, file_type in pairs(music_types) do
    local file_list = gui.scan_directory(JUKEBOX.search_dir, "*." .. file_type)
    for k, v in pairs(file_list) do
      table.insert(music_dir, v)
    end
  end
  if not music_dir then
    gui.printf("Failed to scan directory '%s'\nPlease create it and place some music files in it!\n\n", JUKEBOX.search_dir)
    return
  end
  -- pad the music table if not enough music files
  while #music_dir < #music_list do
    for k, v in pairs(table.copy(music_dir)) do
        table.insert(music_dir, v)
      end
  end
  rand.shuffle(music_dir)
  gui.printf("\nTotal music tracks to choose from: %s\nReplacement table:\n\n", #music_dir)
  local language_lump = {"[default]\n"}
  local used_file = ""
  for i = 1, #music_list do
    for l = 1, #language_lump do
      if string.find(language_lump[l], music_list[i]) then
        used_file = string.sub(language_lump[l], 16, #language_lump[l]-3)
        break
      end
    end
    local track_name
    if used_file == "" then
      gui.wad_insert_file(JUKEBOX.search_dir .. "/" .. music_dir[i], music_prefix .. music_list[i])
      track_name = music_dir[i]
      for _, file_type in pairs(music_types) do
        track_name = string.gsub(track_name, "%." .. file_type .. "$", "")
      end
    else
      track_name = used_file
      used_file = ""
	end
    -- detect map name/number
    local map_num
    if (OB_CONFIG.game == "doom1") or (OB_CONFIG.game == "ultdoom") then
      map_num = music_list[i]
    elseif (OB_CONFIG.game == "heretic") then
      map_num = JUKEBOX.heretic_levels[i]
    else
      if i < 10 then
        map_num = "MAP0" .. i
      else
        map_num = "MAP" .. i
      end
    end
    -- add the track name to the "now playing" list
    gui.printf(map_num .. ": " .. music_list[i] .. " -> " .. track_name .. "\n")
    table.insert(language_lump, "_MUSIC_" .. map_num .. " = \"" .. track_name .. "\";\n")
  end
  -- add a "now playing" message on level start using ACS
  gui.wad_add_binary_lump("A_START", {})
  gui.wad_insert_file(JUKEBOX.script_dir .. "/JUKEBOX.o", "JUKEBOX")
  gui.wad_add_binary_lump("A_END", {})
  gui.wad_insert_file(JUKEBOX.script_dir .. "/JUKEBOX.acs", "JUKE_SRC")
  gui.wad_add_text_lump("LOADACS", {"JUKEBOX\n"})
  gui.wad_add_text_lump("LANGUAGE", language_lump)
  -- add a console command to repeat the message
  gui.wad_add_text_lump("KEYCONF", {"alias nowplaying \"pukename PrintNowPlaying\"\n"})
end


OB_MODULES["jukebox"] =
{
  label = _("Jukebox"),

  side = "left",
  priority = 80,

  game = {doom1 = 1, ultdoom = 1, doom2 = 1, tnt = 1, plutonia = 1, heretic = 1},

  port = "any",
  tooltip=_("Replace all music tracks in the WAD with tracks from a variety of sources. Do not use with any other music randomizer."),

  hooks =
  {
    get_levels = JUKEBOX.insert_music,
  },
}

